package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.core.util.SpringContextHolder;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.ApplicationForUseBatchDTO;
import digital.laboratory.platform.reagent.dto.ApplicationForUseDTO;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.entity.ApplicationForUse;
import digital.laboratory.platform.reagent.entity.ReagentConsumablesSet;
import digital.laboratory.platform.reagent.service.ApplicationForUseService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesSetService;
import digital.laboratory.platform.reagent.vo.ApplicationForUseVO;
import digital.laboratory.platform.reagent.vo.ReagentConsumablesSetVO;
import org.springframework.beans.BeanUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.util.List;

/**
 * (试剂耗材领用申请表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材领用申请表) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/application_for_use")
@Api(value = "application_for_use", tags = "(试剂耗材领用申请表)管理")
public class ApplicationForUseController {

    private final ApplicationForUseService applicationForUseService;

    private final ReagentConsumablesSetService reagentConsumablesSetService;

    /**
     * 通过id查询(试剂耗材领用申请表)
     *
     * @param applicationForUseId id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping()
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_get')")
    public R<ApplicationForUseVO> getById(String applicationForUseId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ApplicationForUseVO applicationForUseVO = applicationForUseService.getApplicationForUseVO(applicationForUseId);

        return R.ok(applicationForUseVO);
    }

    /**
     * 分页查询
     *
     * @param page              分页对象
     * @param applicationForUse (试剂耗材领用申请表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_page')")
    public R<IPage<ApplicationForUseVO>> getApplicationForUsePage(Page<ApplicationForUse> page, ApplicationForUse applicationForUse, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<ApplicationForUseVO> applicationForUseVOList = applicationForUseService.getApplicationForUseVOPage(page, Wrappers.<ApplicationForUse>query().eq("create_by", dlpUser.getId()).eq(applicationForUse.getStatus()!=null,"status",applicationForUse.getStatus()).orderByDesc("create_time"));

        return R.ok(applicationForUseVOList);

    }

    /**
     * 新增(试剂耗材领用申请表)
     *
     * @param applicationForUseDTO (试剂耗材领用申请表)
     * @return R
     */
    @ApiOperation(value = "新增(试剂耗材领用申请表)", notes = "新增(试剂耗材领用申请表)")
    @SysLog("新增(试剂耗材领用申请表)")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_add')")
    public R<ApplicationForUseVO> postAddObject(@RequestBody ApplicationForUseDTO applicationForUseDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ApplicationForUseVO applicationForUse = applicationForUseService.addApplication(applicationForUseDTO, dlpUser);

        if (applicationForUse != null) {
            return R.ok(applicationForUse, "保存成功");
        } else return R.failed("保存失败");
    }

    /**
     * 修改(试剂耗材领用申请表)
     *
     * @param applicationForUseDTO (试剂耗材领用申请表)
     * @return R
     */
    @ApiOperation(value = "修改(试剂耗材领用申请表)", notes = "修改(试剂耗材领用申请表)")
    @SysLog("修改(试剂耗材领用申请表)")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_edit')")
    public R<String> putUpdateById(@RequestBody ApplicationForUseDTO applicationForUseDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReagentConsumablesSet byId = reagentConsumablesSetService.getById(applicationForUseDTO.getReagentConsumableSetId());

        BeanUtils.copyProperties(applicationForUseDTO, byId);

        if (reagentConsumablesSetService.updateById(byId)) {
            return R.ok("修改成功");
        } else {
            return R.failed("修改失败");
        }
    }


    /**
     * 通过id删除(试剂耗材领用申请表)
     *
     * @param applicationForUseId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(试剂耗材领用申请表)", notes = "通过id删除(试剂耗材领用申请表)")
    @SysLog("通过id删除(试剂耗材领用申请表)")
    @DeleteMapping()
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_del')")
    public R<String> deleteById(String applicationForUseId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (applicationForUseService.delApplication(applicationForUseId)) {
            return R.ok("删除成功");
        } else return R.failed("删除失败");
    }

    /**
     * 通过id删除(试剂耗材领用申请表)
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除(试剂耗材领用申请表)明细", notes = "通过id删除(试剂耗材领用申请表)明细")
    @SysLog("通过id删除(试剂耗材领用申请表)明细")
    @DeleteMapping("/details")
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_del')")
    public R<String> deleteDetails(String id, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (reagentConsumablesSetService.removeById(id)) {
            return R.ok("删除成功");
        } else return R.failed("删除失败");
    }

    /**
     * 提交试剂耗材领用申请表
     *
     * @param id (试剂耗材领用申请表)
     * @return R
     */
    @ApiOperation(value = "提交试剂耗材领用申请表", notes = "提交试剂耗材领用申请表")
    @SysLog("修改试剂耗材领用申请表")
    @PostMapping("/commit")
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_commit')")
    public R<ApplicationForUseVO> commitById(String id, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ApplicationForUseVO applicationForUse = applicationForUseService.commitApplication(id, dlpUser);

        if (applicationForUse != null) {
            return R.ok(applicationForUse, "提交成功");
        } else return R.failed("提交失败");
    }

    /**
     * 批量提交试剂耗材领用申请表
     *
     * @param applicationForUseBatchDTO (批量提交试剂耗材领用申请表)
     * @return R
     */
    @ApiOperation(value = "批量提交试剂耗材领用申请表", notes = "批量提交试剂耗材领用申请表")
    @SysLog("批量提交试剂耗材领用申请表")
    @PostMapping("/batch/commit")
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_commit')")
    public R<ApplicationForUseVO> batchCommit(@RequestBody ApplicationForUseBatchDTO applicationForUseBatchDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ApplicationForUseVO applicationForUse = applicationForUseService.batchCommit(applicationForUseBatchDTO, dlpUser);

        if (applicationForUse != null) {
            return R.ok(applicationForUse, "提交成功");
        } else return R.failed("提交失败");
    }

    /**
     * 批量提交试剂耗材领用申请表
     *
     * @param id (批量提交试剂耗材领用申请表)
     * @return R
     */
    @ApiOperation(value = "通过实验ID查询领用情况", notes = "通过实验ID查询领用情况")
    @SysLog("通过实验ID查询领用情况")
    @GetMapping("/get/experiment")
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_get')")
    public R<List<ReagentConsumablesSetVO>> getReagentConsumablesSets(String id, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        List<ReagentConsumablesSetVO> reagentConsumablesSets = applicationForUseService.getReagentConsumablesSets(id);

        if (reagentConsumablesSets != null) {
            return R.ok(reagentConsumablesSets);
        } else return R.failed("未能查询到相关信息");
    }
}
