package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AcceptanceRecordFormAuditDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.utils.PageUtils;
import digital.laboratory.platform.reagent.dto.AcceptanceRecordFormDTO;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.mapper.AcceptanceRecordFormMapper;
import digital.laboratory.platform.reagent.vo.AcceptanceRecordFormFullVO;
import digital.laboratory.platform.reagent.vo.AcceptanceRecordFormVO;
import digital.laboratory.platform.reagent.vo.WarehousingContentVO;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

/**
 * (验收记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (验收记录表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class AcceptanceRecordFormServiceImpl extends ServiceImpl<AcceptanceRecordFormMapper, AcceptanceRecordForm> implements AcceptanceRecordFormService {

    @Autowired
    private BlacklistService blacklistService;
    @Autowired
    private AcceptanceSignForService acceptanceSignForService;
    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private WarehousingContentService warehousingContentService;
    @Autowired
    private OssFile ossFile;
    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;
    @Autowired
    private WarehousingRecordFormService warehousingRecordFormService;

    @Transactional
    @Override//批量提交
    public void culkCommit(List<AcceptanceRecordFormDTO> acceptanceRecordFormDTOList, DLPUser dlpUser) {

        for (AcceptanceRecordFormDTO acceptanceRecordFormDTO : acceptanceRecordFormDTOList) {
            //调用单个提交方法
            AcceptanceRecordForm acceptanceRecordForm = this.commitForm(acceptanceRecordFormDTO, dlpUser);
        }
    }

    @Override//提交验收记录表
    public AcceptanceRecordForm commitForm(AcceptanceRecordFormDTO acceptanceRecordFormDTO, DLPUser dlpUser) {

        AcceptanceRecordForm byId = this.getById(acceptanceRecordFormDTO.getAcceptanceRecordFormId());
        //验收记录状态为-2，代表验收审核不通过后重新提交，需置空原有审核信息
        if (byId.getStatus() == -2) {
            BeanUtils.copyProperties(acceptanceRecordFormDTO, byId);
            byId.setUserName(dlpUser.getName());
            byId.setAuditResultOfPrimary(false);
            byId.setAuditOpinionOfPrimary("");
            byId.setAuditTimeOfPrimary(null);
            byId.setPrimaryAuditorId("");
            byId.setCommitTime(LocalDateTime.now());
            byId.setStatus(1);
            byId.setDateOfAcceptance(LocalDateTime.now());
            byId.setCreateBy(dlpUser.getId());
            if (this.updateById(byId)) {
                return byId;
            } else {
                throw new RuntimeException(String.format("重新提交失败"));
            }
        }
        if (byId.getStatus() == -3) {

            BeanUtils.copyProperties(acceptanceRecordFormDTO, byId);
            byId.setUserName(dlpUser.getName());
            byId.setAuditResultOfSecondary(false);
            byId.setAuditOpinionOfSecondary("");
            byId.setAuditTimeOfSecondary(null);
            byId.setSecondaryAuditorId("");
            byId.setAuditResultOfPrimary(false);
            byId.setAuditOpinionOfPrimary("");
            byId.setAuditTimeOfPrimary(null);
            byId.setPrimaryAuditorId("");
            byId.setCommitTime(LocalDateTime.now());
            byId.setStatus(1);
            byId.setDateOfAcceptance(LocalDateTime.now());
            byId.setCreateBy(dlpUser.getId());
            if (this.updateById(byId)) {
                return byId;
            } else {
                throw new RuntimeException(String.format("重新提交失败"));
            }
        }
        if (byId.getStatus() == -4) {

            BeanUtils.copyProperties(acceptanceRecordFormDTO, byId);
            byId.setUserName(dlpUser.getName());
            byId.setAuditResultOfThreeLevel(false);
            byId.setAuditOpinionOfThreeLevel("");
            byId.setAuditTimeOfThreeLevel(null);
            byId.setThreeLevelAuditorId("");
            byId.setAuditResultOfSecondary(false);
            byId.setAuditOpinionOfSecondary("");
            byId.setAuditTimeOfSecondary(null);
            byId.setSecondaryAuditorId("");
            byId.setAuditResultOfPrimary(false);
            byId.setAuditOpinionOfPrimary("");
            byId.setAuditTimeOfPrimary(null);
            byId.setPrimaryAuditorId("");
            byId.setCommitTime(LocalDateTime.now());
            byId.setStatus(1);
            byId.setDateOfAcceptance(LocalDateTime.now());
            byId.setCreateBy(dlpUser.getId());
            if (this.updateById(byId)) {
                return byId;
            } else {
                throw new RuntimeException(String.format("重新提交失败"));
            }
        }
        //正常提交
        BeanUtils.copyProperties(acceptanceRecordFormDTO, byId);
        byId.setUserName(dlpUser.getName());
        byId.setCommitTime(LocalDateTime.now());
        byId.setStatus(1);
        byId.setCreateBy(dlpUser.getId());
        byId.setDateOfAcceptance(LocalDateTime.now());
        if (this.updateById(byId)) {
            return byId;
        } else {
            throw new RuntimeException(String.format("提交失败"));
        }
    }

    @Override
    public AcceptanceRecordForm addForm(String reagentConsumableId, String supplierId, String acceptanceSignForId) {

        AcceptanceRecordForm acceptanceRecordForm = new AcceptanceRecordForm();

        acceptanceRecordForm.setId(IdWorker.get32UUID().toUpperCase());
        acceptanceRecordForm.setSupplierId(supplierId);//供应商ID
        acceptanceRecordForm.setAcceptanceSignForId(acceptanceSignForId);//验收分类ID
        acceptanceRecordForm.setReagentConsumableId(reagentConsumableId);//验收试剂耗材ID

        ReagentConsumables byId = reagentConsumablesService.getById(reagentConsumableId);

        acceptanceRecordForm.setReagentConsumableName(byId.getReagentConsumableName());//将物品名称赋值给验收记录表
        acceptanceRecordForm.setStatus(0);//初始化状态

        if ((this.save(acceptanceRecordForm))) {

            return acceptanceRecordForm;

        } else throw new RuntimeException(String.format("创建验收任务失败"));
    }

    @Override
    public IPage<AcceptanceRecordFormVO> getAcceptanceRecordFormVOPage(IPage<AcceptanceRecordForm> page, QueryWrapper<AcceptanceRecordForm> qw) {

        IPage<AcceptanceRecordFormVO> acceptanceRecordFormVOPage = baseMapper.getAcceptanceRecordFormVOPage(page, qw);//获取验收记录列表

        List<AcceptanceRecordFormVO> records = acceptanceRecordFormVOPage.getRecords();
        //添加试剂耗材信息至VO
        for (AcceptanceRecordFormVO record : records) {

            ReagentConsumables byId = reagentConsumablesService.getById(record.getReagentConsumableId());

            record.setReagentConsumables(byId);
        }
        return acceptanceRecordFormVOPage;

    }

//    @Override
//    public IPage<AcceptanceRecordFormVO> getAcceptanceRecordFormVORecordPage(IPage<AcceptanceRecordForm> page) {
//
//        LambdaQueryWrapper<AcceptanceRecordForm> acceptanceRecordFormLambdaQueryWrapper = new LambdaQueryWrapper<>();
//
//        acceptanceRecordFormLambdaQueryWrapper.eq(AcceptanceRecordForm::getStatus, 4);
//
//        List<AcceptanceRecordForm> list = this.list(acceptanceRecordFormLambdaQueryWrapper);
//
//        ArrayList<AcceptanceRecordFormVO> acceptanceRecordFormVOS = new ArrayList<>();
//
//        if (list.size() != 0) {
//
//            for (AcceptanceRecordForm acceptanceRecordForm : list) {
//
//                AcceptanceRecordFormVO acceptanceRecordFormVO = this.getAcceptanceRecordFormVO(acceptanceRecordForm.getId());
//
//                acceptanceRecordFormVOS.add(acceptanceRecordFormVO);
//            }
//        }
//        PageUtils pageUtils = new PageUtils();
//
//        Page pages = pageUtils.getPages((int) page.getPages(), (int) page.getSize(), acceptanceRecordFormVOS);
//
//        return pages;
//    }

    @Override
    public AcceptanceRecordFormVO getAcceptanceRecordFormVO(String acceptanceRecordFormId) {

        AcceptanceRecordFormVO acceptanceRecordFormVO = baseMapper.getAcceptanceRecordFormVO(acceptanceRecordFormId);//查询单个验收记录表

        if (acceptanceRecordFormVO == null) {
            return null;
        }

        ReagentConsumables byId = reagentConsumablesService.getById(acceptanceRecordFormVO.getReagentConsumableId());//查询试剂耗材信息

        acceptanceRecordFormVO.setReagentConsumables(byId);

        return acceptanceRecordFormVO;
    }

    @Override//一级审核
    @Transactional
    public boolean primaryAudit(AcceptanceRecordFormAuditDTO acceptanceRecordFormAuditDTO, DLPUser dlpUser) {
        //批量审核
        List<String> uuId = acceptanceRecordFormAuditDTO.getUuId();//验收记录ID数组

        List<AcceptanceRecordForm> acceptanceRecordForms = new ArrayList<>();//验收记录表集合

        for (String id : uuId) {

            AcceptanceRecordForm byId = this.getById(id);

            if (byId.getStatus() != 1) {
                throw new RuntimeException(String.format("当前状态不能审核"));
            }
            byId.setPrimaryAuditorId(dlpUser.getId());//审核人ID
            byId.setAuditOpinionOfPrimary(acceptanceRecordFormAuditDTO.getAuditOpinion());//审核意见
            byId.setAuditTimeOfPrimary(LocalDateTime.now());//审核时间，默认为接口执行时间
            byId.setAuditResultOfPrimary(acceptanceRecordFormAuditDTO.getAuditResult());//审核结果

            if (acceptanceRecordFormAuditDTO.getAuditResult() == true) {
                byId.setStatus(2);//审核成功状态
            } else byId.setStatus(-2);//审核失败状态

            acceptanceRecordForms.add(byId);
        }
        if (this.updateBatchById(acceptanceRecordForms)) {
            return true;
        } else throw new RuntimeException(String.format("审核失败"));
    }


    @Override//二级审核
    @Transactional
    public boolean secondaryAudit(AcceptanceRecordFormAuditDTO acceptanceRecordFormAuditDTO, DLPUser dlpUser) {
        //批量审核
        List<String> uuId = acceptanceRecordFormAuditDTO.getUuId();//验收记录ID数组
        List<AcceptanceRecordForm> acceptanceRecordForms = new ArrayList<>();//验收记录表集合

        for (String id : uuId) {

            AcceptanceRecordForm byId = this.getById(id);

            if (byId.getStatus() != 2) {
                throw new RuntimeException(String.format("当前状态不能审核"));
            }
            byId.setSecondaryAuditorId(dlpUser.getId());//审核人ID
            byId.setAuditOpinionOfSecondary(acceptanceRecordFormAuditDTO.getAuditOpinion());//审核意见
            byId.setAuditTimeOfSecondary(LocalDateTime.now());//审核时间，默认为接口执行时间
            byId.setAuditResultOfSecondary(acceptanceRecordFormAuditDTO.getAuditResult());//审核结果

            if (acceptanceRecordFormAuditDTO.getAuditResult() == true) {
                byId.setStatus(3);//审核成功状态
            } else byId.setStatus(-3);//审核失败状态
            acceptanceRecordForms.add(byId);
        }
        if (this.updateBatchById(acceptanceRecordForms)) {
            return true;
        } else throw new RuntimeException(String.format("审核失败"));
    }


    @Override//三级审核
    @Transactional
    public boolean threeLevelAudit(AcceptanceRecordFormAuditDTO acceptanceRecordFormAuditDTO, DLPUser dlpUser) {
        //批量审核
        List<String> uuId = acceptanceRecordFormAuditDTO.getUuId();//验收记录ID数组
        List<AcceptanceRecordForm> acceptanceRecordForms = new ArrayList<>();//验收记录表集合

        List<String> stringList = new ArrayList<>();//用于接收验收集合表ID

        for (String s : uuId) {
            AcceptanceRecordForm byId = this.getById(s);
            if (!stringList.contains(byId.getAcceptanceSignForId())) {//添加验收集合表ID至数组
                stringList.add(byId.getAcceptanceSignForId());
            }

            if (byId.getStatus() != 3) {
                throw new RuntimeException(String.format("当前状态不能审核"));
            }
            byId.setThreeLevelAuditorId(dlpUser.getId());//审核人ID
            byId.setAuditOpinionOfThreeLevel(acceptanceRecordFormAuditDTO.getAuditOpinion());//审核意见
            byId.setAuditTimeOfThreeLevel(LocalDateTime.now());//审核时间
            byId.setAuditResultOfThreeLevel(acceptanceRecordFormAuditDTO.getAuditResult());//审核结果

            if (acceptanceRecordFormAuditDTO.getAuditResult() == true) {//审核通过
                if (byId.getNonconformingItem() != null) {//验收不合格项目不为空，会写入供应商黑名单
                    blacklistService.addListById2(byId.getReagentConsumableId(), byId.getSupplierId());
                }
                byId.setStatus(6);
            } else byId.setStatus(-4);
            acceptanceRecordForms.add(byId);
        }
        if (this.updateBatchById(acceptanceRecordForms)) {
            //判断是否验收完毕，更改验收集合表状态
            for (String s : stringList) {

                AcceptanceSignFor acceptanceSignFor = acceptanceSignForService.getById(s);//批量审核的验收记录关联的所有验收集合表
                //得到签收物品集合
                List<WarehousingContent> warehousingContentList = warehousingContentService.list(Wrappers.<WarehousingContent>query().eq("warehousing_record_form_id", warehousingRecordFormService.getById(acceptanceSignFor.getWarehousingRecordFormId()).getId()));
                //得到验收集合中的所有验收记录表
                List<AcceptanceRecordForm> recordFormList = this.list(Wrappers.<AcceptanceRecordForm>query().eq("acceptance_sign_for_id", s));

                if (warehousingContentList.size() == recordFormList.size()) {//判断是否签收完毕（签收完成一条物品，才会生成一个验收记录表）

                    int x = 0;

                    for (AcceptanceRecordForm acceptanceRecordForm : recordFormList) {

                        if (acceptanceRecordForm.getStatus() == 6) {//验收审核通过的状态
                            x = x + 1;
                        }
                    }
                    if (x == recordFormList.size()) {//验收审核通过的数量==总验收数量，验收集合项目完成
                        AcceptanceSignFor byId = acceptanceSignForService.getById(s);
                        byId.setStatus(2);
                        acceptanceSignForService.updateById(byId);
                    }
                }
            }
            return true;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override
    public void acceptanceRecordFormTablePDF(AcceptanceRecordFormVO acceptanceRecordFormVO, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("acceptanceRecordFormTablePDF.................");

        ReagentConsumables byId = reagentConsumablesService.getById(acceptanceRecordFormVO.getReagentConsumableId());

        if (acceptanceRecordFormVO.getTheSameBrandAndModel().equals("合格")) {
            acceptanceRecordFormVO.setTheSameBrandAndModelPrint(true);
        } else {
            acceptanceRecordFormVO.setTheSameBrandAndModelPrint(false);

        }
        if (acceptanceRecordFormVO.getConsistentQuantity().equals("合格")) {
            acceptanceRecordFormVO.setConsistentQuantityPrint(true);
        } else {
            acceptanceRecordFormVO.setConsistentQuantityPrint(false);

        }
        if (acceptanceRecordFormVO.getPackingInGoodCondition().equals("合格")) {
            acceptanceRecordFormVO.setPackingInGoodConditionPrint(true);
        } else {
            acceptanceRecordFormVO.setPackingInGoodConditionPrint(false);
        }
        if (acceptanceRecordFormVO.getValidityPeriod().equals("合格")) {
            acceptanceRecordFormVO.setValidityPeriodPrint(true);
        } else {
            acceptanceRecordFormVO.setValidityPeriodPrint(false);
        }
        if (acceptanceRecordFormVO.getDeliveryCycle().equals("合格")) {
            acceptanceRecordFormVO.setDeliveryCyclePrint(true);
        } else {
            acceptanceRecordFormVO.setDeliveryCyclePrint(false);
        }

        String id = acceptanceRecordFormVO.getId();

        String applyFileName = null;
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        if (byId.getCategory().equals("试剂") | byId.getCategory().equals("耗材")) {

            ossFile.fileGet("template" + "/" + "试剂耗材验收记录表模板.docx", bos);

            applyFileName = "试剂耗材验收记录表" + id;
        } else {

            ossFile.fileGet("template" + "/" + "标准物质验收记录表模板.docx", bos);

            applyFileName = "标准物质验收记录表" + id;
        }


        //-----------------------------
        // 生成 word 版本的 验收记录表

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("theSameBrandAndModelPrint", acceptanceRecordFormVO.getTheSameBrandAndModelPrint());
                    put("reagentConsumableName", acceptanceRecordFormVO.getReagentConsumableName());
                    put("dateOfAcceptance", LocalDateTimeUtil.format(acceptanceRecordFormVO.getDateOfAcceptance(), "yyyy年MM月dd日"));
                    put("supplierName", acceptanceRecordFormVO.getSupplierName());
                    put("consistentQuantityPrint", acceptanceRecordFormVO.getConsistentQuantityPrint());
                    put("packingInGoodConditionPrint", acceptanceRecordFormVO.getPackingInGoodConditionPrint());
                    put("validityPeriodPrint", acceptanceRecordFormVO.getValidityPeriodPrint());
                    put("deliveryCyclePrint", acceptanceRecordFormVO.getDeliveryCyclePrint());
                    put("nonconformingItem", acceptanceRecordFormVO.getNonconformingItem());
                    put("followUpTreatment", acceptanceRecordFormVO.getFollowUpTreatment());
                    put("acceptanceConclusion", acceptanceRecordFormVO.getAcceptanceConclusion());
                    put("secondaryAuditorName", acceptanceRecordFormVO.getSecondaryAuditorName());
                    put("auditOpinionOfSecondary", acceptanceRecordFormVO.getAuditOpinionOfSecondary());
                    put("auditTimeOfSecondary", LocalDateTimeUtil.format(acceptanceRecordFormVO.getAuditTimeOfSecondary(), "yyyy年MM月dd日"));
                    put("threeLevelAuditorName", acceptanceRecordFormVO.getThreeLevelAuditorName());
                    put("auditOpinionOfThreeLevel", acceptanceRecordFormVO.getAuditOpinionOfThreeLevel());
                    put("auditTimeOfThreeLevel", LocalDateTimeUtil.format(acceptanceRecordFormVO.getAuditTimeOfThreeLevel(), "yyyy年MM月dd日"));
                    put("year", year);
                    put("bam_remarks", acceptanceRecordFormVO.getBam_remarks());
                    put("cq_remarks", acceptanceRecordFormVO.getCq_remarks());
                    put("pgc_remarks", acceptanceRecordFormVO.getPgc_remarks());
                    put("vp_remarks", acceptanceRecordFormVO.getVp_remarks());
                    put("dc_remarks", acceptanceRecordFormVO.getDc_remarks());
                    put("primaryAuditorName", acceptanceRecordFormVO.getPrimaryAuditorName());
                    put("auditOpinionOfPrimary", acceptanceRecordFormVO.getAuditOpinionOfPrimary());
                    put("auditTimeOfPrimary", LocalDateTimeUtil.format(acceptanceRecordFormVO.getAuditTimeOfPrimary(), "yyyy年MM月dd日"));
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "acceptanceRecordForm" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }


}
