package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PeriodVerificationPlanDTO;
import digital.laboratory.platform.reagent.entity.CheckSchedule;
import digital.laboratory.platform.reagent.entity.PeriodVerificationPlan;
import digital.laboratory.platform.reagent.mapper.CheckScheduleMapper;
import digital.laboratory.platform.reagent.service.CheckScheduleService;
import digital.laboratory.platform.reagent.service.PeriodVerificationImplementationService;
import digital.laboratory.platform.reagent.service.PeriodVerificationPlanService;
import digital.laboratory.platform.reagent.vo.CheckScheduleVO;
import digital.laboratory.platform.reagent.vo.PeriodVerificationPlanVO;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

/**
 * 服务实现类
 *
 * @author Zhang Xiaolong created at  2023-04-10
 * @describe 服务实现类
 */
@Service
@SuppressWarnings("all")
public class CheckScheduleServiceImpl extends ServiceImpl<CheckScheduleMapper, CheckSchedule> implements CheckScheduleService {

    @Autowired
    private PeriodVerificationImplementationService periodVerificationImplementationService;
    @Autowired
    private OssFile ossFile;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;
    @Autowired
    private PeriodVerificationPlanService periodVerificationPlanService;

    @Override
    @Transactional//新增计划
    public CheckScheduleVO addPlan(List<PeriodVerificationPlanDTO> periodVerificationPlanDTOS, DLPUser dlpUser) {

        CheckSchedule checkSchedule = new CheckSchedule();

        checkSchedule.setId(IdWorker.get32UUID().toUpperCase());
        checkSchedule.setManagerId(dlpUser.getId());
        checkSchedule.setStatus(0);

        List<PeriodVerificationPlan> periodVerificationPlans = new ArrayList<>();

        for (PeriodVerificationPlanDTO periodVerificationPlanDTO : periodVerificationPlanDTOS) {

            PeriodVerificationPlan periodVerificationPlan = new PeriodVerificationPlan();

            BeanUtils.copyProperties(periodVerificationPlanDTO, periodVerificationPlan);

            periodVerificationPlan.setCheckScheduleId(checkSchedule.getId());

            periodVerificationPlan.setDeviationAndUncertainty(periodVerificationPlanDTO.getDeviationOrUncertainty());

            periodVerificationPlans.add(periodVerificationPlan);
        }

        if (this.save(checkSchedule) && periodVerificationPlanService.saveBatch(periodVerificationPlans)) {

            CheckScheduleVO checkScheduleVO = this.getCheckScheduleVO(checkSchedule.getId());

            return checkScheduleVO;
        } else throw new RuntimeException(String.format("保存失败"));
    }


    @Override
    @Transactional
    public CheckSchedule editPlan(List<PeriodVerificationPlanDTO> periodVerificationPlanDTOS) {

        CheckSchedule byId = this.getById(periodVerificationPlanDTOS.get(0).getCheckScheduleId());

        LambdaQueryWrapper<PeriodVerificationPlan> periodVerificationPlanLambdaQueryWrapper = new LambdaQueryWrapper<>();

        periodVerificationPlanLambdaQueryWrapper.eq(PeriodVerificationPlan::getCheckScheduleId, byId.getId());

        List<PeriodVerificationPlan> list = periodVerificationPlanService.list(periodVerificationPlanLambdaQueryWrapper);

        periodVerificationPlanService.removeBatchByIds(list);

        List<PeriodVerificationPlan> periodVerificationPlans = new ArrayList<>();

        for (PeriodVerificationPlanDTO periodVerificationPlanDTO : periodVerificationPlanDTOS) {

            PeriodVerificationPlan periodVerificationPlan = new PeriodVerificationPlan();

            BeanUtils.copyProperties(periodVerificationPlanDTO, periodVerificationPlan);

            periodVerificationPlan.setDeviationAndUncertainty(periodVerificationPlanDTO.getDeviationOrUncertainty());

            periodVerificationPlan.setCheckScheduleId(byId.getId());

            periodVerificationPlans.add(periodVerificationPlan);
        }

        if (this.updateById(byId) && periodVerificationPlanService.saveBatch(periodVerificationPlans)) {

            return byId;
        } else throw new RuntimeException(String.format("保存失败"));
    }


    @Override
    @Transactional
    public CheckSchedule commitPlan(List<PeriodVerificationPlanDTO> periodVerificationPlanDTOS, DLPUser dlpUser) {

        CheckSchedule byId = this.getById(periodVerificationPlanDTOS.get(0).getCheckScheduleId());

        if (byId == null) {

            CheckSchedule checkSchedule = this.addPlan(periodVerificationPlanDTOS, dlpUser);
            checkSchedule.setStatus(1);
            checkSchedule.setCommitTime(LocalDateTime.now());
            this.updateById(checkSchedule);

            return checkSchedule;

        } else {

            if (byId.getStatus() == -2) {

                CheckScheduleVO checkScheduleVO = this.addPlan(periodVerificationPlanDTOS, dlpUser);

                return checkScheduleVO;
            }


            CheckSchedule checkSchedule = this.editPlan(periodVerificationPlanDTOS);

            checkSchedule.setStatus(1);
            checkSchedule.setCommitTime(LocalDateTime.now());

            this.updateById(checkSchedule);

            return checkSchedule;
        }

    }

    @Override
    public CheckSchedule auditPlan(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        CheckSchedule byId = this.getById(auditAndApproveDTO.getUuId());

        byId.setAuditOpinionOfTechnical(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditResultOfTechnical(auditAndApproveDTO.getAuditResult());
        byId.setAuditTimeOfTechnical(LocalDateTime.now());
        byId.setTechnicalDirectorId(dlpUser.getId());

        if (auditAndApproveDTO.getAuditResult() == true) {

            LambdaQueryWrapper<PeriodVerificationPlan> periodVerificationPlanLambdaQueryWrapper = new LambdaQueryWrapper<>();

            periodVerificationPlanLambdaQueryWrapper.eq(PeriodVerificationPlan::getCheckScheduleId, byId.getId());

            List<PeriodVerificationPlan> list = periodVerificationPlanService.list(periodVerificationPlanLambdaQueryWrapper);

            for (PeriodVerificationPlan periodVerificationPlan : list) {

                periodVerificationImplementationService.addById(periodVerificationPlan);

            }
            byId.setStatus(6);
        } else {
            byId.setStatus(-2);
        }

        if (this.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override
    public IPage<CheckScheduleVO> getCheckScheduleVOPage(IPage<CheckSchedule> page, QueryWrapper<CheckSchedule> qw) {

        IPage<CheckScheduleVO> checkScheduleVOPage = baseMapper.getCheckScheduleVOPage(page, qw);

        return checkScheduleVOPage;
    }

    @Override
    public CheckScheduleVO getCheckScheduleVO(String checkScheduleId) {

        CheckScheduleVO checkScheduleVO = baseMapper.getCheckScheduleVO(checkScheduleId);

        List<PeriodVerificationPlanVO> periodVerificationPlanVOList = periodVerificationPlanService.getPeriodVerificationPlanVOList(checkScheduleVO.getId());

        checkScheduleVO.setPeriodVerificationPlanVOS(periodVerificationPlanVOList);

        return checkScheduleVO;
    }

    @Override
    public void checkScheduleTablePDF(CheckScheduleVO checkScheduleVO, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("checkScheduleTablePDF.................");

        String id = checkScheduleVO.getId();

        List<PeriodVerificationPlanVO> voList = checkScheduleVO.getPeriodVerificationPlanVOS();
        int x = 1;
        for (PeriodVerificationPlanVO periodVerificationPlanVO : voList) {
            periodVerificationPlanVO.setI(x);
            if (!StringUtils.isNotBlank(periodVerificationPlanVO.getDeviationAndUncertainty())) {
                periodVerificationPlanVO.setDeviationAndUncertainty("/");
            }
            if (!StringUtils.isNotBlank(periodVerificationPlanVO.getStandardValueOrPurity())) {
                periodVerificationPlanVO.setStandardValueOrPurity("/");
            }
            if (periodVerificationPlanVO.getVerificationResult() != null && periodVerificationPlanVO.getVerificationResult().equals("P")) {
                periodVerificationPlanVO.setP("√");
            } else if (periodVerificationPlanVO.getVerificationResult() != null && periodVerificationPlanVO.getVerificationResult().equals("D")) {
                periodVerificationPlanVO.setD("√");
            } else if (periodVerificationPlanVO.getVerificationResult() != null && periodVerificationPlanVO.getVerificationResult().equals("F")) {
                periodVerificationPlanVO.setF("√");
            }
            x++;
        }

        String applyFileName = "标准物质期间核查计划和确认表" + id;
        //-----------------------------
        // 生成 word 版本的 验收记录表

        ByteArrayOutputStream bos = new ByteArrayOutputStream();

        ossFile.fileGet("template" + "/" + "标准物质期间核查计划和确认表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);

        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);

        builder.bind("voList", policy).build();

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("voList", voList);
                    put("year", year);
                    put("managerName", checkScheduleVO.getManagerName());
                    put("technicalDirectorName", checkScheduleVO.getTechnicalDirectorName());
                    put("auditTimeOfTechnical", LocalDateTimeUtil.format(checkScheduleVO.getAuditTimeOfTechnical(), "yyyy年MM月dd日"));

                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "checkSchedule" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }
}
